// https://www.shadertoy.com/view/ldjBDm
// inspired by and copied from iq:
// https://www.shadertoy.com/view/Xd2GR3

#define TWO_PI 6.283185

vec3 iq_color_palette(vec3 a, vec3 b, vec3 c, vec3 d, float t)
{
    return a + b * cos(TWO_PI * (c*t + d));
}

// copied from iq
// { 2d cell id, distance to border, distance to center )
vec4 hexagon( vec2 p ) 
{
	vec2 q = vec2( p.x*2.0*0.5773503, p.y + p.x*0.5773503 );
	
	vec2 pi = floor(q);
	vec2 pf = fract(q);

	float v = mod(pi.x + pi.y, 3.0);

	float ca = step(1.0,v);
	float cb = step(2.0,v);
	vec2  ma = step(pf.xy,pf.yx);
	
    // distance to borders
	float e = dot( ma, 1.0-pf.yx + ca*(pf.x+pf.y-1.0) + cb*(pf.yx-2.0*pf.xy) );

	// distance to center	
	p = vec2( q.x + floor(0.5+p.y/1.5), 4.0*p.y/3.0 )*0.5 + 0.5;
	float f = length( (fract(p) - 0.5)*vec2(1.0,0.85) );		
	
	return vec4( pi + ca - cb*ma, e, f );
}

// copied from iq
float hash1( vec2  p ) { float n = dot(p,vec2(127.1,311.7) ); return fract(sin(n)*43758.5453); }

// copied from iq
float noise( in vec3 x )
{
    vec3 p = floor(x);
    vec3 f = fract(x);
	f = f*f*(3.0-2.0*f);
	vec2 uv = (p.xy+vec2(37.0,17.0)*p.z) + f.xy;
	vec2 rg = textureLod( iChannel0, (uv+0.5)/256.0, 0.0 ).yx;
	return mix( rg.x, rg.y, f.z );
}


void mainImage( out vec4 fragColor, in vec2 fragCoord ) 
{
    vec2 uv = fragCoord.xy/iResolution.xy;
	vec2 pos = (-iResolution.xy + 2.0*fragCoord.xy)/iResolution.y;
	
    float t = .8 * (1. - pow(uv.y, .5));
    pos.y += -.5;
	pos *= 2. - 0.3*pos.y;
    
    vec4 h = vec4(0.);
    float n = 0.;
    vec3 col = vec3(0.);

	h = hexagon(12.0*pos);
    n = noise( vec3(0.3*h.xy+vec2(iTime*.7, 0.),0.) );
    t += 0.2 * (1. - n);
    
	col = 0.9 * abs(sin( hash1(h.xy)*.4 + 1.6 + vec3(1.) ));
	col *= smoothstep( 0.0+t, 0.1+t, h.z );
	col *= 1. + 0.5*h.z*n;        	
    col *= 1. + .8 * iq_color_palette(vec3(0.5, 0.5, 0.5),
                                 vec3(0.5, 0.5, 0.5),
                                 vec3(2.0, 1.0, 0.0),
                                 vec3(0.5, 0.2, 0.25),
                                 iTime * .02);
    col *= .4 + .6 * iq_color_palette(vec3(.5, .5, .5),
                                      vec3(.5, .5, .5),
                                      vec3(.3, 1., 1.),
                                      vec3(0., .25, .25),                                      
	        uv.y-.4-iTime*.1);
	col *= pow( 16.0*uv.x*(1.0-uv.x)*uv.y*(1.0-uv.y), 0.6 );
	
	fragColor = vec4( col, 1.0 );
}