// https://www.shadertoy.com/view/cstcWB

float noise(vec2 p) {
    return fract(sin(dot(p, vec2(256.315, 568.146)))*38554.14159);
}

vec3 getColorFromNoise(vec2 p) {
    float r = noise(p + vec2(13.31, 57.87));
    float g = noise(p + vec2(45.32, 98.21));
    float b = noise(p + vec2(78.12, 23.54));
    return vec3(r, g, b);
}

vec3 sampleGradient(float t) {
    vec3 colorStart = vec3(1.000,0.000,0.000); // Example: light blue
    vec3 colorEnd = vec3(0.067,0.000,1.000);   // Example: darker blue
    return mix(colorStart, colorEnd, t);
}

float getBrightnessFromNoise(vec2 p) {
    return mix(0.5, 1.5, noise(p + vec2(89.65, 41.23)));
}

void mainImage( out vec4 fragColor, in vec2 fragCoord )
{
    
    vec2 uv = (fragCoord-.5*iResolution.xy)/iResolution.y;
    uv.y -=.5;
    uv.y/=1.1;
    uv.x*=1000.;
    float t = iTime/70.;

    vec2 fuv = floor(uv);
    float brightness = 0.9/getBrightnessFromNoise(fuv);
    float wNoise = noise(fuv);
    
    float speedVariation = mix(0.5, 1.5, noise(fuv + vec2(12.34, 56.78))); // Different offsets to get a new noise value
    wNoise -= t * speedVariation;
    
    vec2 sub = fract(uv - vec2(wNoise));
    sub.y = .24 - sub.y;
    float rain = sub.y;
    float glow = smoothstep(.22, 1., rain)*50.;
    rain = clamp(rain, 0., 1.);  
    
    float gradientFactor = fract(fuv.x * 1000.3); // Adjust the multiplier for gradient frequency
    vec3 RrainColor = getColorFromNoise(fuv)*.2;
    
    vec3 rainColor = sampleGradient(gradientFactor);
    rainColor = (rainColor + RrainColor)/2.;
    
    vec3 col = rain * rainColor * brightness* 2.3;
    
    float thresholdForGlow = 1.6; // You can adjust this value to suit your needs
    if (brightness > thresholdForGlow) {
        float glow = smoothstep(.22, 1., rain) * 50.;
        col += glow * 6.;
    }

    
    col /= 1.0;

    fragColor = vec4(col, 1.0);
}